package com.handheldgroup.lrbt_cap;

import android.app.ProgressDialog;
import android.os.Bundle;
import android.os.RemoteException;
import android.os.ServiceManager;
import android.os.SystemClock;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.IOException;

import device.service.IDeviceService;

public class MainActivity extends SerialActivity {
    private static final String TAG = "LRBT";

    private IDeviceService mDeviceService;
    private IDeviceService getDeviceService() {
        if (mDeviceService == null) {
            mDeviceService = IDeviceService.Stub.asInterface(ServiceManager.getService("DeviceService"));
        }
        return mDeviceService;
    }

    ArrayAdapter<DeviceDiscovery> adapter;
    ProgressDialog progressDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        progressDialog = new ProgressDialog(this);
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.setCancelable(false);
        progressDialog.setCanceledOnTouchOutside(false);

        adapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, android.R.id.text1);
        ((ListView)findViewById(R.id.list)).setAdapter(adapter);
    }

    public void onConnectClick(View view) {
        progressDialog.setMessage("Connecting...");
        progressDialog.show();
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    // Enable power on the top port
                    getDeviceService().setExpansion3p3vPower(false);
                    SystemClock.sleep(600);
                    getDeviceService().setExpansion3p3vPower(true);
                    SystemClock.sleep(600);
                    // Connecte to the port
                    connect("/dev/ttyO0", 57600);
                    SystemClock.sleep(600);
                    // Change to AT mode so the module receives our commands
                    changeMode(true);
                    SystemClock.sleep(600);
                    // Get Local name
                    write("AT*AGLN?\r");

                } catch (RemoteException | IOException e) {
                    e.printStackTrace();
                }
            }
        }).start();
    }

    public void onScanClick(View view) {
        try {
            progressDialog.setMessage("Scanning...");
            progressDialog.show();
            adapter.clear();
            // AT*AGDD=<inquiry_type>,<inquiry_length>
            write("AT*AGDD=2,4\r");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void onDataReceived(final String data) {
        Log.i(TAG, "processData: " + data);
        if ("OK".equals(data) || "ERROR".equals(data)) {
            Log.i(TAG, "Status: " + data);
        } else if (data.startsWith("*") && data.contains(":")) {
            String key = data.substring(0, data.indexOf(":"));
            final String value = data.substring(data.indexOf(":") + 1).replace("\"", "").trim();
            switch (key) {
                case "*AGDDE":
                    // Received a device from discovery
                    String[] parts = value.split(",");
                    final DeviceDiscovery d = new DeviceDiscovery(
                            parts[3].replace("\"", ""), // Name
                            parts[0], // Address
                            Integer.valueOf(parts[1]), // COD
                            "1".equals(parts[2]) // isValid
                    );
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            adapter.add(d);
                        }
                    });
                    break;
                case "*AGDD":
                    // Discovery has ended with x number of devices
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(MainActivity.this, value + " device(s) found", Toast.LENGTH_LONG).show();
                            progressDialog.hide();
                        }
                    });
                    break;
                case "*AGLN":
                    // Received value for local name
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            Toast.makeText(MainActivity.this, "Connected to LRBT", Toast.LENGTH_LONG).show();
                           setTitle("Name: " + value);
                            if(progressDialog != null && progressDialog.isShowing())  progressDialog.hide();
                            findViewById(R.id.button_connect).setEnabled(false);
                            findViewById(R.id.button_scan).setEnabled(true);
                        }
                    });
                    break;
                default:
                    Log.i(TAG, "configuration received - " + key + "=" + value);
            }
        }
    }

    @Override
    protected void onDestroy() {
        try {
            disconnect();
        } catch (IOException e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }

    public void changeMode(boolean data) throws IOException {
        if (data) {
            SystemClock.sleep(1100);
            write("///");
            SystemClock.sleep(1100);
        } else {
            write("AT*ADDM");
        }

    }

    public void onPowerClick(View view) {
        try {
            getDeviceService().setExpansion3p3vPower(false);
            SystemClock.sleep(500);
            getDeviceService().setExpansion3p3vPower(true);
            SystemClock.sleep(500);
            Toast.makeText(this, "Power restarted", Toast.LENGTH_SHORT).show();
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
}
