package com.handheldgroup.lrbt_cap;

import android.app.Activity;
import android.os.SystemClock;
import android.util.Log;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import device.utils.serial.SerialPort;

public abstract class SerialActivity extends Activity {
    private static final String TAG = SerialActivity.class.getSimpleName();

    private SerialPort mSerialPort;
    private InputStream mInputStream;
    private ReadThread mReadThread;

    private class ReadThread extends Thread {
        StringBuilder receiveBuffer = new StringBuilder();
        Pattern pattern = Pattern.compile("(.+)\\r\\n");

        @Override
        public void run() {
            super.run();
            while(!isInterrupted()) {
                try {
                    if (mInputStream == null){
                        return;
                    }
                    int read = mInputStream.read();
                    if(read > 0){
                        receiveBuffer.append((char)read);

                        boolean loop = true;
                        do{
                            String in = receiveBuffer.toString();
                            Matcher matcher = pattern.matcher(in);
                            if(matcher.find() && matcher.groupCount() == 1){
                                String strData = matcher.group(0);
                                onDataReceived(strData);
                                receiveBuffer.delete(matcher.start(0), matcher.end(0));
                            } else {
                                loop = false;
                            }
                        }while (loop);
                    }
                } catch (IOException e) {
                    Log.e(TAG, e.getMessage(), e);
                    e.printStackTrace();
                    return;
                }
            }
        }
    }

    public abstract void onDataReceived(String data);

    protected void connect(String path, int baud) throws IOException {
        mSerialPort = new SerialPort(new File(path), baud, 0);
        mInputStream = mSerialPort.getInputStream();
        mReadThread = new ReadThread();
        mReadThread.start();
        SystemClock.sleep(100);
    }

    protected void disconnect() throws IOException {
        mReadThread.interrupt();
        mReadThread = null;
        mInputStream.close();
        mSerialPort.close();
    }

    protected void write(String data) throws IOException {
        write(data.getBytes());
    }

    protected void write(byte[] data) throws IOException {
        mSerialPort.getOutputStream().write(data);
    }
}
